/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
/*
jGuard is a security framework based on top of jaas (java authentication and authorization security).
it is written for web applications, to resolve simply, access control problems.
version $Name$
http://sourceforge.net/projects/jguard/

Copyright (C) 2004  Charles GAY

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


jGuard project home page:
http://sourceforge.net/projects/jguard/

*/
package net.sf.jguard.ext.authentication.manager;

import java.security.Principal;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;
import javax.security.auth.Subject;
import net.sf.jguard.core.Filter;
import net.sf.jguard.core.Filterable;
import net.sf.jguard.core.authentication.AuthenticationException;
import net.sf.jguard.core.authentication.credentials.JGuardCredential;
import net.sf.jguard.core.organization.Organization;
import net.sf.jguard.core.principals.RolePrincipal;
import net.sf.jguard.ext.organization.PersistedOrganization;
import net.sf.jguard.core.provisioning.OrganizationTemplate;
import net.sf.jguard.ext.authentication.PersistedPrincipal;
import net.sf.jguard.ext.authentication.PersistedSubject;
import net.sf.jguard.ext.filters.HibernateFilter;
import net.sf.jguard.ext.util.SubjectUtils;
import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.Example;

/**
* Hibernate AuthenticationManager implementation.
* @author <a href="mailto:diabolo512@users.sourceforge.net">Charles Gay</a>
*/
public class HibernateAuthenticationManager extends AbstractAuthenticationManager implements Filterable{

    private static Logger logger = Logger.getLogger(HibernateAuthenticationManager.class.getName());
    private SessionFactory sessionFactory = null;
    public final static String SESSION_FACTORY="sessionFactory";
    private Filter filter = null;
    
    public HibernateAuthenticationManager(Map options){
        super(options);
        if(options.get(SESSION_FACTORY)!=null && !"".equals(options.get(SESSION_FACTORY))){
            sessionFactory= (SessionFactory)options.get(SESSION_FACTORY);
        }else{
            if(HibernateUtil.getConfiguration()==null){
                HibernateUtil.init();    
            }
            this.sessionFactory = HibernateUtil.getSessionFactory();
        }
        filter = new HibernateFilter(sessionFactory);
        Session session = sessionFactory.getCurrentSession();
        Transaction tx = null;
            try{
                 tx = session.beginTransaction();
                if(isEmpty()){
                importXmlData(options);
                }        
                tx.commit();
            }catch(Exception ex){
                if (tx!=null) tx.rollback();
                throw new RuntimeException(ex);
            }finally{
                if(session != null && session.isOpen()){
                    session.close();
                }
            }
        
        
            
        
    }
    
     
    protected void persistUser(Subject user) throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        
        PersistedOrganization pOrga = getPersistedOrganizationFromSubject(user);
        net.sf.jguard.ext.authentication.PersistedSubject persistedSubject = new net.sf.jguard.ext.authentication.PersistedSubject(user,pOrga);
        session.saveOrUpdate(persistedSubject);
         if(persistedSubject.getId()!=null && !persistedSubject.getId().toString().equals("0")){
            JGuardCredential persistanceIdCredential = new JGuardCredential(net.sf.jguard.ext.authentication.PersistedSubject.PERSISTENCE_ID,persistedSubject.getId().toString());
            user.getPrivateCredentials().add(persistanceIdCredential);
        } 
        user = persistedSubject.toJavaxSecuritySubject();
    }

    protected void persistPrincipal(Principal principal) throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        
        net.sf.jguard.ext.authentication.PersistedPrincipal ppal = HibernateConverterUtils.getPersistedPrincipal(principal);
        if(ppal!=null){
            session.saveOrUpdate(ppal);
        }
        
    }

    protected void persistOrganization(Organization organization) throws AuthenticationException {
        PersistedOrganization orga = null;
        orga = new PersistedOrganization(organization);
        
        Session session = sessionFactory.getCurrentSession();
        session.saveOrUpdate(orga);
        organization.setId(orga.getId());
        
    }

    protected void updateUserImpl(JGuardCredential identityCred, Subject user) throws AuthenticationException {
        PersistedOrganization pOrga = getPersistedOrganizationFromSubject(user);
        Session session = sessionFactory.getCurrentSession();
        Long id = new Long((SubjectUtils.getCredentialValueAsString(user, false, PersistedSubject.PERSISTENCE_ID)));
        PersistedSubject subject = (PersistedSubject)session.get(PersistedSubject.class,id);
        subject.setPrivateCredentials(user.getPrivateCredentials());
        subject.setPublicCredentials(user.getPublicCredentials());
        Set principals = user.getPrincipals();
        Iterator it = principals.iterator();
        Set persistedPrincipals = new HashSet();
        while(it.hasNext()){
            Principal ppal = (Principal)it.next();
            if(ppal instanceof RolePrincipal){
                Long idPpal = ((RolePrincipal)ppal).getId();
                PersistedPrincipal persistedPrincipal = (PersistedPrincipal)session.get(PersistedPrincipal.class, idPpal);
                persistedPrincipals.add(persistedPrincipal);
            }
        }
        subject.setPrincipals(persistedPrincipals);
        session.update(subject);
    }

    public Set findUsers(Collection privateCredentials,Collection publicCredentials) throws AuthenticationException {
        Set usersFound = new HashSet();
        
        //use public credentials
        Iterator itPrivate = privateCredentials.iterator();
        while(itPrivate.hasNext()){
            JGuardCredential cred = (JGuardCredential)itPrivate.next();
            List users = findUsers(cred, true);
            usersFound.addAll(users);
        }
        
        //use private credentials
        Iterator itPublic = publicCredentials.iterator();
        while(itPublic.hasNext()){
            JGuardCredential cred = (JGuardCredential)itPublic.next();
            List users = findUsers(cred, false);
            usersFound.addAll(users);
        }
      
        return usersFound;
    }
    
    private List findUsers(JGuardCredential cred, boolean priv){
        Session session = sessionFactory.getCurrentSession();
        String q=" select subject from net.sf.jguard.ext.authentication.PersistedSubject as subject ";
        if(priv){
                   q+=" inner join subject.privateCredentials as cred ";
        }else{
            q+=" inner join subject.publicCredentials as cred ";
        }
            
            q+= " where cred.name= :name and cred.value= :value ";
        Query query = session.createQuery(q);
        query.setString("name", cred.getName());
        query.setString("value", cred.getValue().toString());
        
        List results = query.list();
        return results;
        
    }
    
     public Subject findUser(String login){
        Session session = sessionFactory.getCurrentSession();
        
        String q=" select subject from net.sf.jguard.ext.authentication.PersistedSubject as subject " +
                   " inner join subject.privateCredentials as priv " +
                   " where priv.name='"+getCredentialId()+"' and priv.value= :login ";
        Query query = session.createQuery(q);
        query.setString("login", login);
        net.sf.jguard.ext.authentication.PersistedSubject result = (net.sf.jguard.ext.authentication.PersistedSubject)query.uniqueResult();
        if(result==null){
            return null;
        }
        Set set = new HashSet();
        set.add(result);
        Set s = SubjectUtils.getJavaxSecuritySubjects(set);
        Iterator it= s.iterator();
        Subject subject = (Subject)it.next();
        return subject;
     }

    public Set getUsers() throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        Criteria criteria = session.createCriteria(net.sf.jguard.ext.authentication.PersistedSubject.class);
        List usersList = criteria.list();
        Set users = new HashSet(usersList);
        Set javaxSecuritySubjects = SubjectUtils.getJavaxSecuritySubjects(users);
        return javaxSecuritySubjects;
    }

    public Collection findOrganizations(Collection credentials) throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        Criteria criteria = session.createCriteria(net.sf.jguard.ext.authentication.PersistedSubject.class);
        Organization org = new Organization();
        org.setCredentials(new HashSet(credentials));
        Example example = Example.create(org);
        criteria.add(example);
        List results = criteria.list();
        return new HashSet(results);
    }

    public void deleteOrganization(Organization organization) {
        Session session = sessionFactory.getCurrentSession();
        session.delete(organization);
    }

    public Set getOrganizations() throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery(" select orga from net.sf.jguard.ext.organization.PersistedOrganization as orga inner join orga.credentials as creds  where creds.name='id' and creds.value!= 'template'");
        List organizationsList = query.list();
        Set orgas = new HashSet(organizationsList);
        orgas = HibernateConverterUtils.getOrganizations(orgas);
        return orgas;
    }

    public void updateOrganization(String organizationIdentityCredential, Organization organization) throws AuthenticationException {
        
            Session session = sessionFactory.getCurrentSession();
            PersistedOrganization pOrg = findPersistedOrganization(organization.getName());
            session.update(pOrg);
        
    }

    public Organization findOrganization(String organizationName) {
        PersistedOrganization persistedOrganization = findPersistedOrganization(organizationName);
        Organization orga = null;
        if(persistedOrganization!=null){
         orga = persistedOrganization.toOrganization();
        }
        
        return orga;
    }

    public Set getAllPrincipalsSet() throws AuthenticationException {
         Session session = sessionFactory.getCurrentSession();
         Criteria principalsCriteria = session.createCriteria(net.sf.jguard.ext.authentication.PersistedPrincipal.class);
         List principals = principalsCriteria.list();
         Set ppals = HibernateConverterUtils.getjavaSecurityPrincipals(new HashSet(principals));
         return ppals;
    }

    public void deleteUser(Subject subject) throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        String idToString = SubjectUtils.getCredentialValueAsString(subject, false,PersistedSubject.PERSISTENCE_ID);
        if(idToString==null|| idToString.equals("")){
            throw new IllegalArgumentException("subject hasn't got any persistenceId. we cannot delete a subject not persisted ");
        }
        PersistedSubject subjectToDelete = (PersistedSubject)session.get(PersistedSubject.class, new Long(idToString));
        
        session.delete(subjectToDelete);
    }

    public boolean isEmpty() {
        
        Session session = sessionFactory.getCurrentSession();
        
            Criteria orgCriteria = session.createCriteria(Organization.class);
            List orgas = orgCriteria.list();
            if(orgas.size()>0){
                return false;
            }
            Criteria subjectCriteria = session.createCriteria(net.sf.jguard.ext.authentication.PersistedSubject.class);
            List subjects = subjectCriteria.list();
            if(subjects.size()>0){
                return false;
            }

            Criteria principalsCriteria = session.createCriteria(net.sf.jguard.ext.authentication.PersistedPrincipal.class);
            List principals = principalsCriteria.list();

            if(principals.size()>0){
                return false;
            }
        return true;
    }

    public void updatePrincipal(String oldPrincipalName, Principal principal) throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        
        net.sf.jguard.ext.authentication.PersistedPrincipal ppal = HibernateConverterUtils.getPersistedPrincipal(principal);
        if(ppal!=null && ppal.getId()!=null){
            session.update(ppal);
        }else{
            logger.warning(" principal to update is not persisted in the database");
        }
        
        
    }

    public boolean deletePrincipal(Principal principal) throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        
        net.sf.jguard.ext.authentication.PersistedPrincipal ppal = HibernateConverterUtils.getPersistedPrincipal(principal);
         if(ppal!=null){
            session.delete(ppal);
         }
        
        return true;
    }

    public void persistUserAccessAttempt(String login, String hostAddress, String hostName, boolean succeeded, long timeStamp) throws AuthenticationException {
        logger.finest("persistUserAccessAttempt is not yet implemented in HibernateAuthenticationManager");
    }

     /**
     * @return OrganizationTemplate.
     */
    public OrganizationTemplate getOrganizationTemplate() {
        
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery(" select orga from net.sf.jguard.ext.organization.PersistedOrganization as orga inner join  orga.credentials  as creds where creds.name='id' and creds.value='template' ");
        PersistedOrganization orga = (PersistedOrganization)query.uniqueResult();
        if(orga == null){
            return null;
        }
        
        OrganizationTemplate orgaTemplate = new OrganizationTemplate(orga.toOrganization());
        
        return orgaTemplate;        
        
    }

    public void setOrganizationTemplate(OrganizationTemplate organizationTemplate) throws AuthenticationException {
        Session session = sessionFactory.getCurrentSession();
        //we grab from the organization list the organizationTemplate identified by id='template'
        Query query = session.createQuery(" select orga from net.sf.jguard.ext.organization.PersistedOrganization as orga inner join orga.credentials as creds where creds.name='id' and creds.value='template' ");
        PersistedOrganization orga = (PersistedOrganization)query.uniqueResult();
        
            if(orga!=null){
                PersistedOrganization convertedOrga = new PersistedOrganization(organizationTemplate.toOrganization());
                //organizationTemplate is already present; we update it
                orga.setCredentials(convertedOrga.getCredentials());
                orga.setPrincipals(convertedOrga.getPrincipals());
                orga.setSubjectTemplate(convertedOrga.getSubjectTemplate());    
                session.update(orga);
            }else{
                PersistedOrganization newOrga = new PersistedOrganization(organizationTemplate.toOrganization());
                session.save(newOrga);
            }
        
    }

    private PersistedOrganization findPersistedOrganization(String organizationName) throws HibernateException, IllegalStateException {
        Session session = sessionFactory.getCurrentSession();
        Query query = session.createQuery(" select organization from net.sf.jguard.ext.organization.PersistedOrganization as organization join organization.credentials as credentials where credentials.name='id' and credentials.value=:organizationId");
        query.setString("organizationId", organizationName);
        List results = query.list();
        PersistedOrganization persistedOrganization = null;
        if (results.size() > 1) {
            throw new IllegalStateException(" more than one organization is identified by " + organizationName);
        } else if (results.size() == 1) {
            persistedOrganization = (PersistedOrganization) results.get(0);
        }
        return persistedOrganization;
    }

    private PersistedOrganization getPersistedOrganizationFromSubject(Subject user) {
        Organization organization = SubjectUtils.getOrganization(user);
        PersistedOrganization pOrga = (PersistedOrganization) findPersistedOrganization(organization.getName());
        return pOrga;
    }

    public Filter getFilter() {
        return filter;
    }
    
    
}
